/**
 * Filename:  	P2SiteRebuilder.java
 * Copyright: 	HSP GmbH - 2011
 * Purpose:   	
 * 
 * Author:    	Oldenburg
 * Revision:   	
 * Created: 	Sep 21, 2011 1:57:45 PM
 */
package com.hsp.tools.buckminster;

import java.io.File;
import java.net.URI;
import java.util.Iterator;

import org.eclipse.buckminster.core.actor.AbstractActor;
import org.eclipse.buckminster.core.actor.IActionContext;
import org.eclipse.buckminster.core.actor.IActor;
import org.eclipse.buckminster.core.actor.MissingPrerequisiteException;
import org.eclipse.buckminster.core.cspec.model.Action;
import org.eclipse.buckminster.core.cspec.model.Attribute;
import org.eclipse.buckminster.core.cspec.model.CSpec;
import org.eclipse.buckminster.core.cspec.model.Prerequisite;
import org.eclipse.buckminster.pde.PDEPlugin;
import org.eclipse.buckminster.runtime.Logger;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.equinox.p2.core.ProvisionException;
import org.eclipse.equinox.p2.internal.repository.tools.RecreateRepositoryApplication;
import org.eclipse.equinox.p2.internal.repository.tools.RepositoryDescriptor;

/**
 * @author Oldenburg
 * 
 */
@SuppressWarnings("restriction")
public class P2SiteRebuilder extends AbstractActor implements IActor {

	private Logger logger = null;
	
	/**
	 * 
	 */
	public P2SiteRebuilder() {
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.buckminster.core.actor.AbstractActor#internalPerform(org.
	 * eclipse.buckminster.core.actor.IActionContext,
	 * org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
	protected IStatus internalPerform(IActionContext ctx, IProgressMonitor monitor) throws CoreException {

		IPath site = null;

		Action action = ctx.getAction();
		CSpec cspec = action.getCSpec();

		for (Iterator<Prerequisite> iterator = action.getPrerequisites().iterator(); iterator.hasNext();) {
			Prerequisite preq = (Prerequisite) iterator.next();
			if ("site".equals(preq.getAlias())) {
				Attribute rt = preq.getReferencedAttribute(cspec, ctx);
				if (rt != null) {
					site = AbstractActor.getSingleAttributePath(ctx, rt, true);
				}
			}
		}

		if (site == null) {
			throw new MissingPrerequisiteException(action, "site");
		}

		File siteFile = site.toFile().getAbsoluteFile();
		logger = PDEPlugin.getLogger();
		
		if (siteFile.isDirectory()) {
			// run p2.processRepoTask
			logger.info("starting re-build on repo: " + siteFile);
			return recreateRepository(siteFile.toURI(), monitor);
		}

		return Status.OK_STATUS;
	}

	private IStatus recreateRepository(URI repository, IProgressMonitor monitor) {
		logger.info("preparing re-build desc on repo: " + repository);
		final RepositoryDescriptor descriptor = new RepositoryDescriptor();
		descriptor.setAppend(true);
		descriptor.setFormat(null);
		descriptor.setKind("artifact");
		descriptor.setLocation(repository);

		try {
			logger.info("preparing re-build app on repo: " + repository);
			RecreateRepositoryApplication application = new RecreateRepositoryApplication();
			application.setArtifactRepository(descriptor);

			logger.info("running re-build on repo: " + repository);
			application.run(monitor);
			logger.info("finished re-build on repo: " + repository);
		} catch (ProvisionException e) {
			e.printStackTrace();
			return Status.CANCEL_STATUS;
		}
		return Status.OK_STATUS;
	}

}
